"""
Control an itach ip2ir gateway using libitachip2ir
"""
from ctypes import *
import os
import fnmatch
import sys
import logging

def findlib():
    dirname = os.path.dirname(__file__)
    for ext in ['so','dylib','dll','pyd']:
        names = fnmatch.filter(os.listdir(dirname),"*itachip2ir*" + ext)
        for name in names:
            try:
                return cdll.LoadLibrary(dirname +"/"+name)
            except OSError:
                pass
    return None
libitachip2ir = findlib()
if libitachip2ir is None:
    raise OSError("Unable to find itachip2ir library")

libitachip2ir.ITachIP2IR_new.argtypes = [c_char_p, c_char_p, c_int]
libitachip2ir.ITachIP2IR_new.restype = c_void_p
libitachip2ir.ITachIP2IR_delete.argtypes = [c_void_p]
libitachip2ir.ITachIP2IR_ready.argtypes = [c_void_p, c_int]
libitachip2ir.ITachIP2IR_ready.restype = c_bool
libitachip2ir.ITachIP2IR_update.argtypes = [c_void_p]
libitachip2ir.ITachIP2IR_addDevice.argtypes = [c_void_p, c_char_p, c_int, c_int, c_char_p]
libitachip2ir.ITachIP2IR_addDevice.restype = c_bool
libitachip2ir.ITachIP2IR_send.argtypes = [c_void_p, c_char_p, c_char_p, c_int]
libitachip2ir.ITachIP2IR_send.restype = c_bool
c_logcb_p = CFUNCTYPE(None, c_char_p)
libitachip2ir.ITachIP2IR_setLog.argtypes = [c_logcb_p]

if sys.version_info >= (3, 0):
    def tochar(data):
        return bytes(data,'utf-8') if data else None
else:
    def tochar(data):
        return str(data) if data else None

import logging
logger = logging.getLogger(__file__)

def logcb(data):
    logger.info(data.decode('utf-8'))

clogcb = c_logcb_p(logcb)
libitachip2ir.ITachIP2IR_setLog(clogcb)

class ITachIP2IR(object):
    def __init__(self, mac, ip, port):
        self.itachip2ir = libitachip2ir.ITachIP2IR_new(tochar(mac), tochar(ip), port)

    def __del__(self):
        libitachip2ir.ITachIP2IR_delete(self.itachip2ir)

    def ready(self, timeout):
        return libitachip2ir.ITachIP2IR_ready(self.itachip2ir, timeout)

    def update(self):
        libitachip2ir.ITachIP2IR_update(self.itachip2ir)

    def addDevice(self, name, modaddr, connaddr, cmddata):
        return libitachip2ir.ITachIP2IR_addDevice(self.itachip2ir, tochar(name), modaddr, connaddr, tochar(cmddata))

    def send(self, device, command, count):
        libitachip2ir.ITachIP2IR_send(self.itachip2ir, tochar(device), tochar(command), count)



"""
Control an itach ip2ir gateway using libitachip2ir in homeassistant
"""

import logging
from . import pyitachip2ir

from homeassistant.const import DEVICE_DEFAULT_NAME
import homeassistant.components.remote as remote
from homeassistant.components.remote import (
    PLATFORM_SCHEMA, DOMAIN, ATTR_DEVICE, ATTR_COMMAND, ATTR_ACTIVITY)

_LOGGER = logging.getLogger(__name__)

CONF_MAC = 'mac'
CONF_IP = 'ip'
CONF_PORT = 'port'
CONF_FILE ='file'
CONF_DEVICES = 'devices'


# pylint: disable=unused-argument
def setup_platform(hass, config, add_devices, discovery_info=None):
    itachip2ir = pyitachip2ir.ITachIP2IR(config.get(CONF_MAC), config.get(CONF_IP), int(config.get(CONF_PORT)))
    devices = []
    for data in config.get(CONF_DEVICES):
        name = data['name']
        modaddr = int(data.get('modaddr',1))
        connaddr = int(data.get('connaddr',1))
        cmddata = open(hass.config.config_dir + "/" + data.get('file'), "r").read()
        itachip2ir.addDevice(name, modaddr, connaddr, cmddata);
        devices.append(ITachIP2IRRemote(itachip2ir, name))
    add_devices(devices, True)


class ITachIP2IRRemote(remote.RemoteDevice):
    def __init__(self, itachip2ir, name):
        self.itachip2ir = itachip2ir
        self._power = False
        self._name = name or DEVICE_DEFAULT_NAME

    @property
    def name(self):
        """Return the name of the remote."""
        return self._name

    @property
    def is_on(self):
        """Return true if device is on."""
        return self._power

    def turn_on(self, **kwargs):
        """Turn the device on."""
        if kwargs[ATTR_ACTIVITY]:
            activity = kwargs[ATTR_ACTIVITY]
        else:
            activity = "ON"
        self._power = True;
        self.itachip2ir.send(self._name,"ON",1)
        self.schedule_update_ha_state()

    def turn_off(self):
        """Turn the device off."""
        self._power = False;
        self.itachip2ir.send(self._name,"OFF",1)
        self.schedule_update_ha_state()

    def send_command(self, **kwargs):
        """Send a command to one device."""
        self.itachip2ir.send(self._name,kwargs[ATTR_COMMAND],1)

    def update(self):
        self.itachip2ir.update()



"""A setuptools based setup module.
See:
https://packaging.python.org/en/latest/distributing.html
https://bitbucket.com/pallindo/itachip2ir
"""

from setuptools import setup, Extension
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

module = Extension('itachip2ir',
                    sources = [
                        'source/ITachIP2IR.cpp',
                        'source/IRCommandParser.cpp'
                    ])

setup(
    name='pyitachip2ir',
    version='0.0.6',
    description='A library for sending IR commands to an ITach IP2IR gateway',
    long_description=long_description,
    url='https://bitbucket.com/pallindo/itachip2ir',
    author='Alan Fischer',
    author_email='alan@lightningtoads.com',
    license='MIT',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Home Automation',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
    keywords='itach ip2ir homeautomation',
    py_modules = ["pyitachip2ir"],
    ext_modules = [module]
)
